
// SPDX-License-Identifier: BSD-3-Clause

#include <pthread.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include "consumer.h"
#include "ring_buffer.h"
#include "packet.h"
#include "utils.h"
#include <stdio.h>

void consumer_thread(so_consumer_ctx_t *ctx)
{
    char buffer[PKT_SZ];
    char log_entry[256];
    ssize_t sz;

    while (1) {
        sz = ring_buffer_dequeue(ctx->producer_rb, buffer, PKT_SZ);
        if (sz <= 0) break;

        struct so_packet_t *pkt = (struct so_packet_t *)buffer;
        int action = process_packet(pkt);
        unsigned long hash = packet_hash(pkt);
        unsigned long timestamp = pkt->hdr.timestamp;

        pthread_mutex_lock(&ctx->lock);
        while (timestamp < ctx->last_processed_timestamp) {
            pthread_cond_wait(&ctx->cond, &ctx->lock);
        }

        int len = snprintf(log_entry, 256, "%s %016lx %lu",
                           RES_TO_STR(action), hash, timestamp);
        write(ctx->output_fd, log_entry, len);

        ctx->last_processed_timestamp = timestamp;
        pthread_cond_broadcast(&ctx->cond);
        pthread_mutex_unlock(&ctx->lock);
    }
}

int create_consumers(pthread_t *tids, int num_consumers, struct so_ring_buffer_t *rb, const char *out_filename)
{
    for (int i = 0; i < num_consumers; i++) {
        so_consumer_ctx_t *ctx = calloc(1, sizeof(so_consumer_ctx_t));
        ctx->producer_rb = rb;
        ctx->output_fd = open(out_filename, O_RDWR | O_CREAT | O_TRUNC, 0666);
        DIE(ctx->output_fd < 0, "open");

        pthread_mutex_init(&ctx->lock, NULL);
        pthread_cond_init(&ctx->cond, NULL);
        ctx->last_processed_timestamp = 0;

        pthread_create(&tids[i], NULL, (void *(*)(void *))consumer_thread, ctx);
    }

    return num_consumers;
}